﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Interactivity;

namespace Behavior_base
{
    // C:\Program Files\Microsoft SDKs\Expression\Blend 3\Interactivity\Libraries\Silverlight\System.Windows.Interactivity.dll

    public class RotateBehavior : Behavior<UIElement>
    {
        public RotateBehavior()
        {
        }

        private RotateTransform rotate = null;
        private TransformGroup group = null;
        private Transform saveOriginal = null;
        private double angle = 0d;
        private Storyboard sbEnter = null;
        private Storyboard sbLeave = null;
        private TimeSpan duration = new TimeSpan(0,0,0,0,300);

        public double Angle
        {
            get { return angle; }
            set { angle = value; }
        }

        public TimeSpan Duration
        {
            get { return duration; }
            set { duration = value; }
        }

        protected override void OnAttached()
        {
            base.OnAttached();
            saveOriginal = AssociatedObject.RenderTransform;
            rotate = new RotateTransform {Angle = 0d };
            group = new TransformGroup();
            group.Children.Add(rotate);
            AssociatedObject.MouseEnter += AssociatedObject_MouseEnter;
            AssociatedObject.MouseLeave += AssociatedObject_MouseLeave;
        }

        void AssociatedObject_MouseLeave(object sender, MouseEventArgs e)
        {
            if (sbEnter!=null) sbEnter.Stop();
            if (sbLeave!=null) sbLeave.Begin();
        }

        void AssociatedObject_MouseEnter(object sender, MouseEventArgs e)
        {
            if (sbLeave!=null) sbLeave.Stop();
            AssociatedObject.RenderTransform = group;
            rotate.CenterX = (double)AssociatedObject.GetValue(FrameworkElement.WidthProperty) / 2;
            rotate.CenterY = (double)AssociatedObject.GetValue(FrameworkElement.HeightProperty) / 2;
            var da = new DoubleAnimation();
            sbEnter = new Storyboard();
            var du = new Duration(duration);
            da.Duration = du;
            sbEnter.Duration = du;
            sbEnter.Children.Add(da);
            da.To = angle;
            Storyboard.SetTarget(da,rotate);
            Storyboard.SetTargetProperty(da,new PropertyPath(RotateTransform.AngleProperty));
            sbEnter.Begin();

            var dal = new DoubleAnimation();
            sbLeave = new Storyboard();
            var dul = new Duration(duration);
            dal.Duration = dul;
            sbLeave.Duration = dul;
            sbLeave.Children.Add(dal);
            dal.From = angle;
            dal.To = 0d;
            Storyboard.SetTarget(dal, rotate);
            Storyboard.SetTargetProperty(dal, new PropertyPath(RotateTransform.AngleProperty));
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            AssociatedObject.RenderTransform = saveOriginal;
        }
    }
}